
function [] = solve_RO_ls()
%This function solves relative orientation from corresponence image
%measurements
%mixed model and general least squares

    format long g
    
    %load data
    param = load('params.dat');    
    i_obs = load('i_obs.dat')
    
    %size of image observation matrix
    s = size(i_obs)
    
    %visualize image observations
    figure(1)
    scatter(i_obs(:,1),i_obs(:,2))
    axis equal
    ax = gca;
    ax.XLim = [-115 115];
    ax.YLim = [-115 115];
    title('Image observations, left image')
    figure(2)
    scatter(i_obs(:,3),i_obs(:,4))
    axis equal
    ax = gca;
    ax.XLim = [-115 115];
    ax.YLim = [-115 115];
    title('Image observations, right image')

    % initialize
    c = param(1);
    bx = 46; %fixed by user
    by = 0.00; %initial approximation
    bz = 0.01; %initial approximation
    o2 = 0.01; %initial approximation
    p2 = 0.01; %initial approximation
    k2 = 0.01; %initial approximation

    iteration = 0 ; 
    max_correction = 10000;
  
    l0 = i_obs;  %initial observations
    l_e = i_obs; %estimates of observations (first round with original observations)
  
    while iteration < 40 && max_correction > 0.000001
        %initialize v vector
        v = zeros(s(1)*4,1); 
        
        %create C matrix
        C = create_C_matrix(c,bx,by,bz,o2,p2,k2,l_e);

        %create D matrix
        D = create_D_matrix(c,bx,by,bz,o2,p2,k2,l_e);

        W = inv(C*C'); %inverse of a weight matrix
        
        %create h vector
        h = create_h_vector(c,bx,by,bz,o2,p2,k2,l0,l_e,v,C);
        
        dp = (D'*W*D)\(D'*W*h); %solution of orientation corrections
        k = W*(h-D*dp); %solution of Lagrange multipliers
        v = C'*k; %solution of image observation corrections
    
        %correction of parameters
        by = by + dp(1);
        bz = bz + dp(2);
        o2 = o2 + dp(3);
        p2 = p2 + dp(4);
        k2 = k2 + dp(5);
        
        %check max correction
        max_correction = max(abs(dp))  
    
        %correct observations
  
        l_e = l_e + reshape(v,s(1),4);
   
        %current iteration round
        iteration = iteration + 1

    end
    
    %correction check (all values should be close to zero
    disp('Sanity check D_T*W*C*v: ')
    disp(D'*W*C*v)

    %sum of squares
    S=v'*v;
    
    %print results 
    disp('Image base components (m):')
    disp(['bx: ' num2str(bx)   ' by: ' num2str(by) ' bz: ' num2str(bz) ])
    disp('Image rotation components (gon):')
    disp(['omega2: ' num2str(rad2gon(o2)) ' phi2: ' num2str(rad2gon(p2)) ' kappa2: ' num2str(rad2gon(k2))])
    disp('Corrected image observations (mm)')
    disp(num2str(l_e))
    disp('Differences between original image observations and estimated ones (mm)')
    disp(num2str(i_obs-l_e))
    disp('Sum of squares:')
    disp(S)
end

function [gon] = rad2gon(rad)
    gon = (rad*400)/(2*pi());
end


function [h] = create_h_vector(c,bx,by,bz,o2,p2,k2,l0,l_e,v, C)
    %We don't take account observation corrections but only G0
    si = size(l0);
    h = zeros(si(1),1);
    for i = 1:si(1)
       h(i) = -coplanarity_equation(c,bx,by,bz,o2,p2,k2,l_e(i,1),l_e(i,2),l_e(i,3),l_e(i,4));
    end
    h = h - C*(reshape(l_e,4*si(1),1) - reshape(l0,4*si(1),1));

end

function [C] = create_C_matrix(c,bx,by,bz,o2,p2,k2,i_obs)
%This function creates a C matrix
%You need to complete this

    si = size(i_obs);
    %initialize matrix
    C = zeros(si(1),4*si(1));
    
    for i = 1:si(1) %go through all observations
        %TODO: create here C matrix row by row
        %be extra careful to get the columns right (see lecture notes)!
        
    end

end

function [D] = create_D_matrix(c,bx,by,bz,o2,p2,k2,i_obs)
    si = size(i_obs);
    %initialize matrix
    D = zeros(si(1),5);
    for i = 1:si(1)
       x1 = i_obs(i,1); 
       y1 = i_obs(i,2);
       x2 = i_obs(i,3);
       y2 = i_obs(i,4);
       
       %TODO: create here D matrix row by row
       
    end
end

%copy all following functions from the partial_derivatives_RO.m, which you
%created in the previous step

function [G] = coplanarity_equation(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end
 
function [G_d_by] = make_G_d_by(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end

function [G_d_bz] = make_G_d_bz(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end
 
function [G_d_o2] = make_G_d_o2(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end
 
function [G_d_p2] = make_G_d_p2(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end
 
function [G_d_k2] = make_G_d_k2(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end
 
function [G_d_x1] = make_G_d_x1(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end

function [G_d_y1] = make_G_d_y1(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end
 
function [G_d_x2] = make_G_d_x2(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end
 
function [G_d_y2] = make_G_d_y2(c,bx,by,bz,o2,p2,k2,x1,y1,x2,y2)

end
 