function [] = space_intersection()
%This function solves 3D coordinates of object points if orientations of
%two stereo images are known

    close all %cleans the table
    params = load('params.dat') %loads orientation parameters
    %camera constant (mm), X0 (mm), Y0 (mm), Z0 (mm), omega (gon), phi (gon),
    %kappa (gon) for both images

    i_obs = load('i_obs.dat') %loads image observations
    % left image x, left image y, right image x, right image y

    s = size(i_obs); %size of image observation matrix

    %initialize result matrix
    ground3Dpoints = zeros(s(1),3);

    for i = 1:s(1) %solve 3D points of all corresponding image observations
        A = make_A_matrix_SI(params,i_obs(i,:)) %make design matrix
        y = make_y_vector_SI(params,i_obs(i,:)) %make observation vector
    
        x = (A'*A)\(A'*y); %solution (linear case i.e. no iteration needed)
        ground3Dpoints(i,:) = x'; %store the results
    end
    
    %display the results
    disp('3D coordinates of ground points:')
    disp(ground3Dpoints)

    %3D illustration    
    figure(1)
    scatter3(ground3Dpoints(:,1),ground3Dpoints(:,2),ground3Dpoints(:,3))
    axis equal
    ax = gca;
    ax.ZLim = [-190 -150];
    title('Measured 3D points (in the model coordinate system)')
    
    
    %sanity check
    [x y] = colli(params(2,:),ground3Dpoints(1,:));
    if abs(x - i_obs(1,3)) > 0.0001 || abs(y - i_obs(1,4)) > 0.004
        disp('There seems to be some bugs in your code...')
    end
    
end


function [R]=rot(o,p,k)
%makes the 3D rotation matrix from angles omega, phi, kappa (notice that the indexing is
%transpose if compared with e.g. Kraus book)
    R=zeros(3,3);
    R(1,1)= cos(p)*cos(k);
    R(2,1)= -cos(p)*sin(k);
    R(3,1)= sin(p);
    R(1,2)= sin(o)*sin(p)*cos(k)+cos(o)*sin(k);
    R(2,2)= -sin(o)*sin(p)*sin(k)+cos(o)*cos(k);
    R(3,2)= -sin(o)*cos(p);
    R(1,3)= -cos(o)*sin(p)*cos(k)+sin(o)*sin(k);
    R(2,3)= cos(o)*sin(p)*sin(k)+sin(o)*cos(k);
    R(3,3)= cos(o)*cos(p); 
end

function [rad]=gon2rad(gon)
%converts gons into radians    
    rad=(2*pi()*gon)/400;   
end

function [x y]=colli(params,g_obs)
%collinearity equations
    c=params(1,1);
    X0=params(1,2);
    Y0=params(1,3);
    Z0=params(1,4);
    o=params(1,5);
    p=params(1,6);
    k=params(1,7);
    r=rot(gon2rad(o),gon2rad(p),gon2rad(k)); 

    X=g_obs(1,1);
    Y=g_obs(1,2);
    Z=g_obs(1,3);
    
    U=r(1,1)*(X-X0)+r(1,2)*(Y-Y0)+r(1,3)*(Z-Z0);
    V=r(2,1)*(X-X0)+r(2,2)*(Y-Y0)+r(2,3)*(Z-Z0);
    W=r(3,1)*(X-X0)+r(3,2)*(Y-Y0)+r(3,3)*(Z-Z0);
    x=-c*U/W;
    y=-c*V/W;
    
end

function [A]=make_A_matrix_SI(params,i_obs)
%You need to complete this function

%Creates A matrix for one corresponding image observation pair
%this is something you need to complete!!

%defining parameters for easier use
    c_1=params(1,1);
    x1=i_obs(1,1);
    y1=i_obs(1,2);
    o_1=params(1,5);
    p_1=params(1,6);
    k_1=params(1,7);
    r1=rot(gon2rad(o_1),gon2rad(p_1),gon2rad(k_1));
    
    c_2=params(2,1);
    x2=i_obs(1,3);
    y2=i_obs(1,4);
    o_2=params(2,5);
    p_2=params(2,6);
    k_2=params(2,7);
    r2=rot(gon2rad(o_2),gon2rad(p_2),gon2rad(k_2));
    
    %initialize A matrix
    A=zeros(4,3); 
    
    %TODO: create here A matrix 
  
    
    
end

function [y]=make_y_vector_SI(params,i_obs) 
%you need to complete this function

%Creates y vector for one corresponding image observation pair
%this is something you need to complete!!

    c1=params(1,1);
    x1=i_obs(1,1);
    y1=i_obs(1,2);
    X0_1=params(1,2);
    Y0_1=params(1,3);
    Z0_1=params(1,4);
    o1=params(1,5);
    p1=params(1,6);
    k1=params(1,7);
    r1=rot(gon2rad(o1),gon2rad(p1),gon2rad(k1));
    
    c2=params(2,1);
    x2=i_obs(1,3);
    y2=i_obs(1,4);
    X0_2=params(2,2);
    Y0_2=params(2,3);
    Z0_2=params(2,4);
    o2=params(2,5);
    p2=params(2,6);
    k2=params(2,7);
    r2=rot(gon2rad(o2),gon2rad(p2),gon2rad(k2));
    
    %initialize y vector
    y=zeros(4,1);
    
    %TODO: create here y vector


end